@@ -1,10 +1,14 @@
 """Define endpoints for interacting with bridges."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from aionotion.bridge.models import Bridge as BridgeModel
-from aionotion.bridge.models import BridgeAllResponse, BridgeGetResponse
+from aionotion.bridge.models import (
+    Bridge as BridgeModel,
+    BridgeAllResponse,
+    BridgeGetResponse,
+)
 
 if TYPE_CHECKING:
     from aionotion.client import Client
@@ -17,15 +21,19 @@ class Bridge:
         """Initialize.
 
         Args:
+        ----
             client: The aionotion client
+
         """
         self._client = client
 
     async def async_all(self) -> list[BridgeModel]:
         """Get all bridges.
 
-        Returns:
+        Returns
+        -------
             A validated API response payload.
+
         """
         response: BridgeAllResponse = await self._client.async_request_and_validate(
             "get", "/base_stations", BridgeAllResponse
@@ -36,10 +44,13 @@ class Bridge:
         """Get a bridge by ID.
 
         Args:
+        ----
             bridge_id: The ID of the bridge to get.
 
         Returns:
+        -------
             A validated API response payload.
+
         """
         response: BridgeGetResponse = await self._client.async_request_and_validate(
             "get", f"/base_stations/{bridge_id}", BridgeGetResponse
