@@ -1,12 +1,16 @@
 """Define a base client for interacting with Notion."""
+
 from __future__ import annotations
 
+import asyncio
+from collections.abc import Callable
 from datetime import datetime
+from http import HTTPStatus
 from typing import Any, TypeVar, cast
 from uuid import uuid4
 
 from aiohttp import ClientSession, ClientTimeout
-from aiohttp.client_exceptions import ClientError
+from aiohttp.client_exceptions import ClientResponseError
 from mashumaro import DataClassDictMixin
 from mashumaro.exceptions import (
     MissingField,
@@ -34,6 +38,7 @@ API_BASE = "https://api.getnotion.com/api"
 DEFAULT_TIMEOUT = 10
 
 NotionBaseModelT = TypeVar("NotionBaseModelT", bound=DataClassDictMixin)
+RefreshTokenCallbackT = Callable[[str], None]
 
 
 def get_token_header_value(access_token: str, refresh_token: str | None) -> str:
@@ -44,11 +49,14 @@ def get_token_header_value(access_token: str, refresh_token: str | None) -> str:
     present.
 
     Args:
+    ----
         access_token: An access token.
         refresh_token: A refresh token (if it exists).
 
     Returns:
+    -------
         The value for the Authorization header.
+
     """
     if refresh_token:
         return f"Bearer {access_token}"
@@ -64,13 +72,18 @@ class Client:
         """Initialize.
 
         Args:
+        ----
             session: An optional aiohttp ClientSession.
             session_name: An optional session name to use for authentication.
+
         """
         self._access_token: str | None = None
         self._access_token_expires_at: datetime | None = None
+        self._refresh_event = asyncio.Event()
+        self._refresh_lock = asyncio.Lock()
         self._refresh_token: str | None = None
-        self._refreshing_access_token = False
+        self._refresh_token_callbacks: list[RefreshTokenCallbackT] = []
+        self._refreshing = False
         self._session = session
         self._session_name = session_name or uuid4().hex
         self.user_uuid: str = ""
@@ -81,6 +94,11 @@ class Client:
         self.system = System(self)
         self.user = User(self)
 
+    @property
+    def refresh_token(self) -> str | None:
+        """Return the refresh token."""
+        return self._refresh_token
+
     def _save_tokens_from_auth_response(
         self,
         auth_response: AuthenticateViaCredentialsResponse
@@ -89,7 +107,9 @@ class Client:
         """Save the authentication and refresh tokens from an auth response.
 
         Args:
-            auth: An API response containing auth info.
+        ----
+            auth_response: An API response containing auth info.
+
         """
         self._access_token = auth_response.auth.jwt
         self._refresh_token = auth_response.auth.refresh_token
@@ -98,14 +118,32 @@ class Client:
         decoded_jwt = decode_jwt(self._access_token)
         self._access_token_expires_at = utc_from_timestamp(decoded_jwt["exp"])
 
+        # Call all refresh token callbacks:
+        for callback in self._refresh_token_callbacks:
+            callback(self._refresh_token)
+
+    def add_refresh_token_callback(
+        self, callback: RefreshTokenCallbackT
+    ) -> Callable[[], None]:
+        """Add a callback to be called when the refresh token is updated."""
+        self._refresh_token_callbacks.append(callback)
+
+        def remove_callback() -> None:
+            """Remove the callback from the list of callbacks."""
+            self._refresh_token_callbacks.remove(callback)
+
+        return remove_callback
+
     async def async_authenticate_from_credentials(
         self, email: str, password: str
     ) -> None:
         """Authenticate via username and password.
 
         Args:
+        ----
             email: The email address of a Notion account.
             password: The account password.
+
         """
         auth_response: AuthenticateViaCredentialsResponse = (
             await self.async_request_and_validate(
@@ -132,36 +170,46 @@ class Client:
         """Authenticate via a refresh token.
 
         Args:
+        ----
             refresh_token: The refresh token to use. If not provided, the refresh token
                 that was used to authenticate the user initially will be used.
 
         Raises:
+        ------
             InvalidCredentialsError: If no refresh token is provided and the user has
                 not been authenticated yet.
+
         """
-        if refresh_token is None and self._refresh_token is None:
-            raise InvalidCredentialsError("No valid refresh token provided")
+        if not refresh_token:
+            refresh_token = self._refresh_token
 
-        if refresh_token:
-            # If a refresh token is explicitly provided, use it:
-            self._refresh_token = refresh_token
+        if not refresh_token:
+            msg = "No valid refresh token provided"
+            raise InvalidCredentialsError(msg)
 
-        assert self._refresh_token is not None
+        async with self._refresh_lock:
+            self._refreshing = True
+            self._refresh_event.clear()
 
-        auth_response: AuthenticateViaRefreshTokenResponse = (
-            await self.async_request_and_validate(
-                "post",
-                f"/auth/{self.user_uuid}/refresh",
-                AuthenticateViaRefreshTokenResponse,
-                headers={"Accept-Version": "2"},
-                json={
-                    "auth": {
-                        "refresh_token": self._refresh_token,
-                    }
-                },
-            )
-        )
-        self._save_tokens_from_auth_response(auth_response)
+            try:
+                auth_response: AuthenticateViaRefreshTokenResponse = (
+                    await self.async_request_and_validate(
+                        "post",
+                        f"/auth/{self.user_uuid}/refresh",
+                        AuthenticateViaRefreshTokenResponse,
+                        refresh_request=True,
+                        headers={"Accept-Version": "2"},
+                        json={
+                            "auth": {
+                                "refresh_token": self._refresh_token,
+                            }
+                        },
+                    )
+                )
+                self._save_tokens_from_auth_response(auth_response)
+            finally:
+                self._refreshing = False
+                self._refresh_event.set()
 
     async def async_legacy_authenticate_from_credentials(
         self, email: str, password: str
@@ -171,8 +219,10 @@ class Client:
         This is kept in place for compatibility, but should be considered deprecated.
 
         Args:
+        ----
             email: The email address of a Notion account.
             password: The account password.
+
         """
         LOGGER.warning(
             "Using legacy authentication endpoint; this is deprecated and will be "
@@ -199,36 +249,47 @@ class Client:
         self,
         method: str,
         endpoint: str,
-        **kwargs: dict[str, Any],
+        *,
+        refresh_request: bool = False,
+        headers: dict[str, str] | None = None,
+        json: dict[str, Any] | None = None,
     ) -> dict[str, Any]:
         """Make an API request.
 
         Args:
+        ----
             method: An HTTP method.
             endpoint: A relative API endpoint.
-            **kwargs: Additional kwargs to send with the request.
+            refresh_request: Whether this is a request to refresh the access token.
+            headers: Additional headers to include in the request.
+            json: A JSON payload to send with the request.
 
         Returns:
+        -------
             An API response payload.
 
         Raises:
+        ------
             InvalidCredentialsError: Raised upon invalid credentials.
             RequestError: Raised upon an underlying HTTP error.
+
         """
-        if (
-            not self._refreshing_access_token
-            and self._access_token_expires_at
-            and utcnow() >= self._access_token_expires_at
-        ):
+        if self._access_token_expires_at and utcnow() >= self._access_token_expires_at:
             LOGGER.debug("Access token expired, refreshing...")
-            self._refreshing_access_token = True
+            self._access_token = None
+            self._access_token_expires_at = None
             await self.async_authenticate_from_refresh_token()
 
+        # If an authenticated request arrives while we're refreshing, hold until the
+        # refresh process is done:
+        if not refresh_request and self._refreshing:
+            await self._refresh_event.wait()
+
         url: str = f"{API_BASE}{endpoint}"
 
-        kwargs.setdefault("headers", {})
+        headers = {}
         if self._access_token:
-            kwargs["headers"]["Authorization"] = get_token_header_value(
+            headers["Authorization"] = get_token_header_value(
                 self._access_token, self._refresh_token
             )
 
@@ -239,23 +300,21 @@ class Client:
 
         data: dict[str, Any] = {}
 
-        async with session.request(method, url, **kwargs) as resp:
+        async with session.request(method, url, headers=headers, json=json) as resp:
             data = await resp.json()
 
             try:
                 resp.raise_for_status()
-            except ClientError as err:
-                if "401" in str(err):
-                    raise InvalidCredentialsError("Invalid credentials") from err
+            except ClientResponseError as err:
+                if resp.status == HTTPStatus.UNAUTHORIZED:
+                    msg = "Invalid credentials"
+                    raise InvalidCredentialsError(msg) from err
                 raise RequestError(data["errors"][0]["title"]) from err
 
         if not use_running_session:
             await session.close()
 
-        LOGGER.debug("Received data from /%s: %s", endpoint, data)
-
-        if self._refreshing_access_token:
-            self._refreshing_access_token = False
+        LOGGER.debug("Received data from %s: %s", endpoint, data)
 
         return data
 
@@ -264,20 +323,34 @@ class Client:
         method: str,
         endpoint: str,
         model: type[DataClassDictMixin],
-        **kwargs: dict[str, Any],
+        *,
+        refresh_request: bool = False,
+        headers: dict[str, str] | None = None,
+        json: dict[str, Any] | None = None,
     ) -> NotionBaseModelT:
         """Make an API request and validate the response against a Pydantic model.
 
         Args:
+        ----
             method: An HTTP method.
             endpoint: A relative API endpoint.
             model: A Pydantic model to validate the response against.
-            **kwargs: Additional kwargs to send with the request.
+            refresh_request: Whether this is a request to refresh the access token.
+            headers: Additional headers to include in the request.
+            json: A JSON payload to send with the request.
 
         Returns:
+        -------
             A parsed, validated Pydantic model representing the response.
+
         """
-        raw_data = await self.async_request(method, endpoint, **kwargs)
+        raw_data = await self.async_request(
+            method,
+            endpoint,
+            refresh_request=refresh_request,
+            headers=headers,
+            json=json,
+        )
 
         try:
             return cast(NotionBaseModelT, model.from_dict(raw_data))
@@ -286,12 +359,11 @@ class Client:
             SuitableVariantNotFoundError,
             UnserializableDataError,
         ) as err:
-            raise RequestError(
-                f"Error while parsing response from {endpoint}: {err}"
-            ) from err
+            msg = f"Error while parsing response from {endpoint}: {err}"
+            raise RequestError(msg) from err
 
 
-async def async_get_client(
+async def async_get_client_with_credentials(
     email: str,
     password: str,
     *,
@@ -299,9 +371,10 @@ async def async_get_client(
     session_name: str | None = None,
     use_legacy_auth: bool = False,
 ) -> Client:
-    """Return an authenticated API object.
+    """Return an authenticated API object (using username/password).
 
     Args:
+    ----
         email: The email address of a Notion account.
         password: The account password.
         session: An optional aiohttp ClientSession.
@@ -309,7 +382,9 @@ async def async_get_client(
         use_legacy_auth: Whether to use the legacy authentication endpoint.
 
     Returns:
+    -------
         An authenticated Client object.
+
     """
     client = Client(session=session, session_name=session_name)
     if use_legacy_auth:
@@ -317,3 +392,34 @@ async def async_get_client(
     else:
         await client.async_authenticate_from_credentials(email, password)
     return client
+
+
+# Alias for backwards compatibility:
+async_get_client = async_get_client_with_credentials
+
+
+async def async_get_client_with_refresh_token(
+    user_uuid: str,
+    refresh_token: str,
+    *,
+    session: ClientSession | None = None,
+    session_name: str | None = None,
+) -> Client:
+    """Return an authenticated API object (using a refresh token).
+
+    Args:
+    ----
+        user_uuid: The UUID of the user.
+        refresh_token: A refresh token.
+        session: An optional aiohttp ClientSession.
+        session_name: An optional session name to use for authentication.
+
+    Returns:
+    -------
+        An authenticated Client object.
+
+    """
+    client = Client(session=session, session_name=session_name)
+    client.user_uuid = user_uuid
+    await client.async_authenticate_from_refresh_token(refresh_token=refresh_token)
+    return client
