@@ -1,10 +1,14 @@
 """Define endpoints for interacting with sensors."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from aionotion.sensor.models import Sensor as SensorModel
-from aionotion.sensor.models import SensorAllResponse, SensorGetResponse
+from aionotion.sensor.models import (
+    Sensor as SensorModel,
+    SensorAllResponse,
+    SensorGetResponse,
+)
 
 if TYPE_CHECKING:
     from aionotion.client import Client
@@ -17,15 +21,19 @@ class Sensor:
         """Initialize.
 
         Args:
+        ----
             client: The aionotion client
+
         """
         self._client = client
 
     async def async_all(self) -> list[SensorModel]:
         """Get all sensors.
 
-        Returns:
+        Returns
+        -------
             A validated API response payload.
+
         """
         response: SensorAllResponse = await self._client.async_request_and_validate(
             "get", "/sensors", SensorAllResponse
@@ -36,10 +44,13 @@ class Sensor:
         """Get a sensor by ID.
 
         Args:
+        ----
             sensor_id: The ID of the sensor to get.
 
         Returns:
+        -------
             A validated API response payload.
+
         """
         response: SensorGetResponse = await self._client.async_request_and_validate(
             "get", f"/sensors/{sensor_id}", SensorGetResponse
