@@ -1,10 +1,14 @@
 """Define endpoints for interacting with systems (accounts)."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from aionotion.system.models import System as SystemModel
-from aionotion.system.models import SystemAllResponse, SystemGetResponse
+from aionotion.system.models import (
+    System as SystemModel,
+    SystemAllResponse,
+    SystemGetResponse,
+)
 
 if TYPE_CHECKING:
     from aionotion.client import Client
@@ -17,15 +21,19 @@ class System:
         """Initialize.
 
         Args:
+        ----
             client: The aionotion client
+
         """
         self._client = client
 
     async def async_all(self) -> list[SystemModel]:
         """Get all systems.
 
-        Returns:
+        Returns
+        -------
             An API response payload.
+
         """
         response: SystemAllResponse = await self._client.async_request_and_validate(
             "get", "/systems", SystemAllResponse
@@ -36,10 +44,13 @@ class System:
         """Get a system by ID.
 
         Args:
+        ----
             system_id: The ID of the system to get.
 
         Returns:
+        -------
             An API response payload.
+
         """
         response: SystemGetResponse = await self._client.async_request_and_validate(
             "get", f"/systems/{system_id}", SystemGetResponse
