@@ -1,4 +1,7 @@
 """Define auth utilities."""
+
+from __future__ import annotations
+
 from typing import Any
 
 import jwt
@@ -8,10 +11,13 @@ def decode_jwt(encoded_jwt: str) -> dict[str, Any]:
     """Decode and return a JWT.
 
     Args:
+    ----
         encoded_jwt: An encoded JWT.
 
     Returns:
+    -------
         A decoded JWT.
+
     """
     return jwt.decode(
         encoded_jwt,
